@@ -1,11 +1,11 @@
 """Blocksat API"""
+import json
 import logging
 import os
 import shlex
 import subprocess
 import textwrap
-from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter, \
-    ArgumentTypeError
+from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter
 from shutil import which
 
 import qrcode
@@ -17,15 +17,18 @@ from .. import defs
 from .demorx import DemoRx
 from .gpg import Gpg, config_keyring
 from .listen import ApiListener
-from .order import ApiOrder
-from .pkt import calc_ota_msg_len, ApiChannel
+from .order import ApiOrder, ApiChannel, \
+    API_CHANNELS,\
+    SENDABLE_API_CHANNELS,\
+    PAID_API_CHANNELS,\
+    ORDER_STATUS,\
+    ORDER_QUEUES
+from .pkt import calc_ota_msg_len
 
 logger = logging.getLogger(__name__)
 server_map = {
     'main': "https://api.blockstream.space",
-    'test': "https://api.blockstream.space/testnet",
-    'gossip': "https://api.blockstream.space/gossip",
-    'btc-src': "https://api.blockstream.space/btc-src"
+    'test': "https://api.blockstream.space/testnet"
 }
 
 
@@ -36,20 +39,6 @@ def _get_server_addr(net, server):
         return server_map[net]
 
 
-def _warn_common_overrides(args):
-    """Warn options overridden by arguments --gossip and --btc-src"""
-    if (not (args.gossip or args.btc_src)):
-        return
-
-    opt = "--gossip" if args.gossip else "--btc-src"
-
-    if (args.channel != ApiChannel.USER.value):
-        logger.warning("Option {} overrides --channel".format(opt))
-
-    if (args.server != server_map['main'] or args.net is not None):
-        logger.warning("Option {} overrides --net and --server".format(opt))
-
-
 def config(args):
     """API config command"""
     gnupghome = os.path.join(args.cfg_dir, args.gnupghome)
@@ -87,7 +76,7 @@ def send(args):
         with open(args.file, 'rb') as f:
             data = f.read()
 
-    assert(len(data) > 0), \
+    assert (len(data) > 0), \
         "Empty {}".format("file" if args.file else "message")
 
     # Put file or text within an API message
@@ -108,17 +97,19 @@ def send(args):
     logger.info("Satellite transmission will use %d bytes" % (tx_len))
 
     # Ask user for bid or take it from argument
-    bid = args.bid if args.bid else bidding.ask_bid(tx_len)
+    if (args.channel in PAID_API_CHANNELS):
+        bid = args.bid if args.bid else bidding.ask_bid(tx_len)
+    else:
+        bid = None
 
     # API transmission order
     order = ApiOrder(server_addr, tls_cert=args.tls_cert, tls_key=args.tls_key)
-    res = order.send(msg.get_data(), bid, args.regions)
+    res = order.send(msg.get_data(), bid, args.regions, args.channel)
     print()
 
-    # The API servers (except the gossip and btc-src instances) should return a
-    # Lightning invoice for the transmission order
-    if (server_addr != server_map['gossip']
-            and server_addr != server_map['btc-src']):
+    # Only the paid API channels return a Lightning invoice for the
+    # transmission order
+    if (args.channel in PAID_API_CHANNELS):
         payreq = res["lightning_invoice"]["payreq"]
 
         # Print QR code
@@ -163,16 +154,19 @@ def listen(args):
     # --btc-src arguments, if present
     if (args.gossip):
         channel = ApiChannel.GOSSIP.value
-        server_addr = server_map['gossip']
         args.plaintext = True
     elif (args.btc_src):
         channel = ApiChannel.BTC_SRC.value
-        server_addr = server_map['btc-src']
         args.plaintext = True
     else:
         channel = args.channel
-        server_addr = _get_server_addr(args.net, args.server)
 
+    if (args.gossip or args.btc_src):
+        if (args.channel != ApiChannel.USER.value):
+            logger.warning("Option {} overrides --channel".format(
+                "--gossip" if args.gossip else "--btc-src"))
+
+    server_addr = _get_server_addr(args.net, args.server)
     historian_cli = os.path.join(args.historian_path, 'historian-cli') \
         if (args.historian_path) else 'historian-cli'
 
@@ -199,8 +193,6 @@ def listen(args):
     else:
         gossip_opts = None
 
-    _warn_common_overrides(args)  # warn args overridden by --gossip/--btc-src
-
     if (args.exec):
         # Do not support the option in plaintext mode
         if (args.plaintext):
@@ -283,6 +275,23 @@ def bump(args):
     qr.print_ascii()
 
 
+def get(args):
+    """Get an API order"""
+    server_addr = _get_server_addr(args.net, args.server)
+
+    # Fetch the ApiOrder
+    order = ApiOrder(server_addr, tls_cert=args.tls_cert, tls_key=args.tls_key)
+    order.get(args.uuid, args.auth_token)
+    print(json.dumps(order.order, indent=4))
+
+
+def list_orders(args):
+    server_addr = _get_server_addr(args.net, args.server)
+    order = ApiOrder(server_addr, tls_cert=args.tls_cert, tls_key=args.tls_key)
+    res = order.get_orders(args.status, args.channel, args.queue, args.limit)
+    print(json.dumps(res, indent=4))
+
+
 def delete(args):
     """Cancel an API order"""
     server_addr = _get_server_addr(args.net, args.server)
@@ -297,21 +306,7 @@ def delete(args):
 
 def demo_rx(args):
     """Demo satellite receiver"""
-
-    # Override some options based on the mutual exclusive --gossip and
-    # --btc-src arguments, if present
-    if (args.gossip):
-        server_addr = server_map['gossip']
-        channel = ApiChannel.GOSSIP.value
-    elif (args.btc_src):
-        server_addr = server_map['btc-src']
-        channel = ApiChannel.BTC_SRC.value
-    else:
-        server_addr = _get_server_addr(args.net, args.server)
-        channel = args.channel
-
-    # Argument validation and special cases
-    _warn_common_overrides(args)  # warn args overridden by --gossip/--btc-src
+    server_addr = _get_server_addr(args.net, args.server)
 
     # Open one socket for each interface:
     socks = list()
@@ -320,22 +315,21 @@ def demo_rx(args):
         sock.set_mcast_tx_opts(args.ttl, args.dscp)
         socks.append(sock)
 
-    rx = DemoRx(server_addr, socks, args.bitrate, args.event, channel,
-                args.regions, args.tls_cert, args.tls_key)
+    rx = DemoRx(server_addr,
+                socks,
+                args.bitrate,
+                args.event,
+                args.channel,
+                args.regions,
+                args.tls_cert,
+                args.tls_key,
+                args.poll,
+                sock_by_region=args.if_by_region)
     rx.run()
 
 
-def subparser(subparsers):
+def subparser(subparsers):  # pragma: no cover
     """Subparser for usb command"""
-
-    def channel_number(value):
-        ivalue = int(value)
-        if ivalue < 0 or ivalue > 255:
-            raise ArgumentTypeError(
-                "Invalid API channel number {}. "
-                "Choose number within [0, 256)".format(value))
-        return ivalue
-
     p = subparsers.add_parser('api',
                               description="Blockstream Satellite API",
                               help='Blockstream Satellite API',
@@ -353,10 +347,8 @@ def subparser(subparsers):
         '--net',
         choices=server_map.keys(),
         default=None,
-        help="Choose between the Mainnet API server (main), the Testnet API \
-        server (test), the receive-only server used for Lightning gossip \
-        messages (gossip), or the receive-only server used for messages \
-        carrying the Bitcoin source code (btc-src)")
+        help="Choose between the Mainnet API server (main) or the Testnet API \
+        server (test)")
     server_addr.add_argument('-s',
                              '--server',
                              default=server_map['main'],
@@ -412,6 +404,13 @@ def subparser(subparsers):
                     default=None,
                     type=int,
                     help="Bid (in millisatoshis) for the message transmission")
+    p2.add_argument(
+        '-c',
+        '--channel',
+        type=int,
+        default=ApiChannel.USER.value,
+        choices=SENDABLE_API_CHANNELS,
+        help="API transmission channel over which the message should be sent.")
     p2.add_argument(
         '--regions',
         nargs="+",
@@ -521,8 +520,9 @@ def subparser(subparsers):
     p3.add_argument(
         '-c',
         '--channel',
+        type=int,
         default=ApiChannel.USER.value,
-        type=channel_number,
+        choices=API_CHANNELS,
         help="Listen to a specific API transmission channel. If set to 0, "
         "listen to all channels. By default, listen to user transmissions, "
         "which are sent on channel 1")
@@ -602,18 +602,16 @@ def subparser(subparsers):
         action="store_true",
         help="Configure the application to receive Lightning gossip snapshots "
         "and load them using the historian-cli application. This argument "
-        "overrides the following options: 1) --plaintext (enabled); 2) "
-        "--channel (set to {}); and 3) --server/--net (set to {})".format(
-            ApiChannel.GOSSIP.value, server_map['gossip']))
+        "overrides the following options: 1) --plaintext (enabled); and 2) "
+        "--channel (set to {}).".format(ApiChannel.GOSSIP.value))
     btc_src_gossip_arg_group1.add_argument(
         '--btc-src',
         default=False,
         action="store_true",
         help="Configure the application to receive API messages carrying the "
         "Bitcoin Satellite and Bitcoin Core source codes. This argument "
-        "overrides the following options: 1) --plaintext (enabled); 2) "
-        "--channel (set to {}); and 3) --server/--net (set to {})".format(
-            ApiChannel.BTC_SRC.value, server_map['btc-src']))
+        "overrides the following options: 1) --plaintext (enabled); and 2) "
+        "--channel (set to {}).".format(ApiChannel.BTC_SRC.value))
     p3.add_argument(
         '--historian-path',
         default=None,
@@ -689,6 +687,12 @@ def subparser(subparsers):
         '''),
         help='Run demo satellite receiver',
         formatter_class=ArgumentDefaultsHelpFormatter)
+    p6.add_argument(
+        '--poll',
+        action='store_true',
+        default=False,
+        help='Poll transmitting messages from the API queue instead of '
+        'listening to server-sent event notifications.')
     p6.add_argument(
         '-d',
         '--dest',
@@ -704,18 +708,26 @@ def subparser(subparsers):
         default=["lo"])
     p6.add_argument('-c',
                     '--channel',
+                    type=int,
                     default=ApiChannel.USER.value,
-                    type=channel_number,
+                    choices=SENDABLE_API_CHANNELS,
                     help="Target API transmission channel")
     p6.add_argument(
         '-r',
         '--regions',
         nargs="+",
-        choices=range(0, 6),
+        choices=range(0, len(defs.satellites)),
         type=int,
         help="Coverage region of the demo receiver. Optionally define "
         "multiple arguments to simulate reception from multiple satellite "
         "beams simultaneously. An empty list implies all regions.")
+    p6.add_argument(
+        '--if-by-region',
+        action='store_true',
+        default=False,
+        help="Map each network interface to a single region. "
+        "Requires the same number of arguments on the -i/--interface "
+        "and -r/--regions options.")
     p6.add_argument('--ttl',
                     type=int,
                     default=1,
@@ -737,25 +749,51 @@ def subparser(subparsers):
                     choices=["transmitting", "sent"],
                     default="sent",
                     help='SSE event that should trigger packet transmissions')
-    btc_src_gossip_arg_group2 = p6.add_mutually_exclusive_group()
-    btc_src_gossip_arg_group2.add_argument(
-        '--gossip',
-        default=False,
-        action="store_true",
-        help="Configure the application to fetch and relay Lightning gossip "
-        "messages. This argument overrides option --channel (set to {}) and "
-        "options --server/--net (set to {})".format(ApiChannel.GOSSIP.value,
-                                                    server_map['gossip']))
-    btc_src_gossip_arg_group2.add_argument(
-        '--btc-src',
-        default=False,
-        action="store_true",
-        help="Configure the application to fetch and relay messages carrying "
-        "the Bitcoin Satellite and Bitcoin Core source codes. This argument "
-        "overrides option --channel (set to {}) and options --server/--net "
-        "(set to {})".format(ApiChannel.BTC_SRC.value, server_map['btc-src']))
     p6.set_defaults(func=demo_rx)
 
+    # Get message
+    p7 = subsubparsers.add_parser(
+        'get',
+        description="Get an API message order",
+        help="Get an API message order",
+        formatter_class=ArgumentDefaultsHelpFormatter)
+    p7.add_argument('-u',
+                    '--uuid',
+                    default=None,
+                    help="Universally unique identifier (UUID)")
+    p7.add_argument('-a',
+                    '--auth-token',
+                    default=None,
+                    help="Authentication token")
+    p7.set_defaults(func=get)
+
+    # List orders
+    p8 = subsubparsers.add_parser(
+        'list',
+        description="List transmission orders on the server queue",
+        help="List transmission orders on the server queue",
+        formatter_class=ArgumentDefaultsHelpFormatter)
+    p8.add_argument('-c',
+                    '--channel',
+                    type=int,
+                    default=ApiChannel.USER.value,
+                    choices=SENDABLE_API_CHANNELS,
+                    help="Order transmission channel")
+    p8.add_argument('--status',
+                    choices=ORDER_STATUS,
+                    default=None,
+                    nargs='+',
+                    help="Target order status")
+    p8.add_argument('--queue',
+                    choices=ORDER_QUEUES,
+                    default='queued',
+                    help="Target order queue")
+    p8.add_argument('--limit',
+                    type=int,
+                    default=20,
+                    help="Limit for the number of orders returned")
+    p8.set_defaults(func=list_orders)
+
     return p
 
 
