@@ -2,6 +2,7 @@ import getpass
 import logging
 import os
 import stat
+import sys
 
 import gnupg
 
@@ -77,7 +78,11 @@ class Gpg():
             return
         if (prompt is None):
             prompt = 'Please enter a passphrase: '
-        self.set_passphrase(getpass.getpass(prompt=prompt))
+        try:
+            self.set_passphrase(getpass.getpass(prompt=prompt))
+        except KeyboardInterrupt:
+            print("\nAborting")
+            sys.exit(1)
 
     def set_passphrase(self, passphrase):
         if (self.passphrase is not None):
@@ -85,6 +90,28 @@ class Gpg():
             return
         self.passphrase = passphrase
 
+    def test_passphrase(self, fingerprint):
+        """Validate the defined key passphrase
+
+        Args:
+            fingerprint (str): Fingerprint of the private key associated with
+                the passphrase.
+
+        Returns:
+            bool: Whether the passphrase works for the given private key.
+
+        """
+        if self.passphrase is None:
+            return False
+        res = self.gpg.export_keys(
+            fingerprint,
+            True,  # private key
+            passphrase=self.passphrase)
+        success = res is not None and res != ''
+        if not success:
+            logger.error("Bad passphrase.")
+        return success
+
     def get_default_public_key(self):
         """Get info corresponding to the first public key on the keyring
 
@@ -114,7 +141,7 @@ class Gpg():
 
         """
         key_map = self.gpg.list_keys().key_map
-        assert(fingerprint in key_map), \
+        assert (fingerprint in key_map), \
             "Could not find public key {}".format(fingerprint)
         return key_map[fingerprint]
 
@@ -126,7 +153,7 @@ class Gpg():
             'uids'.
         """
         key_map = self.gpg.list_keys(True).key_map
-        assert(fingerprint in key_map), \
+        assert (fingerprint in key_map), \
             "Could not find private key {}".format(fingerprint)
         return key_map[fingerprint]
 
@@ -148,7 +175,7 @@ class Gpg():
 
     def sign(self, data, keyid, clearsign=True, detach=False):
         """Sign a given data array"""
-        assert(not (clearsign and detach)), \
+        assert (not (clearsign and detach)), \
             "clearsign and detach options are mutually exclusive"
 
         if (not self.interactive and self.passphrase is None):
@@ -162,7 +189,7 @@ class Gpg():
                              passphrase=self.passphrase)
 
 
-def _is_gpg_keyring_set(gnupghome):
+def is_gpg_keyring_set(gnupghome):
     """Check if the keyring is already configured
 
     It is configured when:
@@ -189,34 +216,16 @@ def _is_gpg_keyring_set(gnupghome):
     return True
 
 
-def config_keyring(gpg, log_if_configured=False):
-    """Configure the local keyring
-
-    Create a keypair and import Blockstream's public key
+def import_bs_pubkey(gpg):
+    """Import Blockstream's public key from the package directory
 
     Args:
-        gpg : Gpg object
-        log_if_configured : Print log message if keyring is already configured
+        gpg (Gpg): Gpg object.
 
+    Raises:
+        RuntimeError: If the GPG keyring is not properly configured before
+        calling this function.
     """
-    assert (isinstance(gpg, Gpg))
-    if _is_gpg_keyring_set(gpg.gpghome):
-        if (log_if_configured):
-            logger.info("Keyring already configured")
-        return
-
-    # Generate new keypair
-    logger.info("Generating a GPG keypair to encrypt and decrypt API messages")
-    name = input("User name represented by the key: ")
-    email = input("E-mail address: ")
-    comment = input("Comment to attach to the user ID: ")
-    gpg.create_keys(name, email, comment)
-
-    # Import Blockstream's public key
-    #
-    # NOTE: the order is important here. Add Blockstream's public key only
-    # after adding the user key. With that, the user key becomes the first key
-    # on the keyring, which is used by default.
     pkg_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
     key_path = os.path.join(pkg_dir, 'gpg', defs.blocksat_pubkey + ".gpg")
 
@@ -233,5 +242,37 @@ def config_keyring(gpg, log_if_configured=False):
 
     gpg.gpg.trust_keys(defs.blocksat_pubkey, 'TRUST_ULTIMATE')
 
-    if (not _is_gpg_keyring_set(gpg.gpghome)):
+    if (not is_gpg_keyring_set(gpg.gpghome)):
         raise RuntimeError("GPG keyring configuration failed")
+
+
+def config_keyring(gpg, log_if_configured=False):
+    """Configure the local keyring
+
+    Create a keypair and import Blockstream's public key
+
+    Args:
+        gpg : Gpg object
+        log_if_configured : Print log message if keyring is already configured
+
+    """
+    assert (isinstance(gpg, Gpg))
+    if is_gpg_keyring_set(gpg.gpghome):
+        if (log_if_configured):
+            logger.info("Keyring already configured")
+        return
+
+    # Generate new keypair
+    logger.info("Generating a GPG keypair to encrypt and decrypt API messages")
+    name = util.string_input("User name represented by the key")
+    email = util.string_input("E-mail address")
+    comment = util.string_input("Comment to attach to the user ID",
+                                optional=True)
+    gpg.create_keys(name, email, comment)
+
+    # Import Blockstream's public key
+    #
+    # NOTE: the order is important here. Add Blockstream's public key only
+    # after adding the user key. With that, the user key becomes the first key
+    # on the keyring, which is used by default.
+    import_bs_pubkey(gpg)
