@@ -3,11 +3,37 @@ import logging
 import requests
 import textwrap
 import time
+from enum import Enum
+
 from . import bidding, pkt
 
 logger = logging.getLogger(__name__)
 
 
+class ApiChannel(Enum):
+    ALL = 0
+    USER = 1
+    AUTH = 3
+    GOSSIP = 4
+    BTC_SRC = 5
+
+
+API_CHANNELS = [x.value for x in ApiChannel]
+API_CHANNEL_SSE_NAME = {
+    ApiChannel.USER.value: 'transmissions',
+    ApiChannel.AUTH.value: 'auth',
+    ApiChannel.GOSSIP.value: 'gossip',
+    ApiChannel.BTC_SRC.value: 'btc-src',
+}
+SENDABLE_API_CHANNELS = [x for x in API_CHANNELS if x != ApiChannel.ALL.value]
+PAID_API_CHANNELS = [ApiChannel.USER.value]
+ORDER_STATUS = [
+    'pending', 'paid', 'transmitting', 'sent', 'received', 'cancelled',
+    'expired', 'confirming'
+]
+ORDER_QUEUES = ['pending', 'queued', 'sent']
+
+
 class ApiOrder:
     """API Transmission Order
 
@@ -32,6 +58,7 @@ class ApiOrder:
         # TLS key/cert
         self.tls_cert = tls_cert
         self.tls_key = tls_key
+        self.admin = tls_cert is not None and tls_key is not None
 
         # Transmission sequence number, if defined
         self.seq_num = seq_num
@@ -82,7 +109,8 @@ class ApiOrder:
         assert (self.uuid is not None)
         assert (self.auth_token is not None)
 
-        r = requests.get(self.server + '/order/' + self.uuid,
+        endpoint = '/admin/order/' if self.admin else '/order/'
+        r = requests.get(self.server + endpoint + self.uuid,
                          headers={'X-Auth-Token': self.auth_token},
                          cert=(self.tls_cert, self.tls_key))
 
@@ -109,7 +137,51 @@ class ApiOrder:
         self.auth_token = auth_token
         self._fetch()
 
-    def send(self, data, bid, regions=None):
+    def get_orders(self, status: list, channel=1, queue='queued', limit=20):
+        """Get API orders with a target status
+
+        Args:
+            status (list): Filter the list of orders to contain orders with the
+                the status in this list only.
+            channel (int): Target channel. Defaults to channel 1.
+            queue (str): Queue from which the order can be fetched (pending,
+                queued, or sent orders). Defaults to the 'queued' queue.
+            limit (int): Maximum number of orders on the result.
+
+        Returns:
+            list: List with the filtered orders.
+
+        """
+        assert queue in ORDER_QUEUES
+        assert status is None or [x in ORDER_STATUS for x in status]
+        endpoint = '/admin/orders/' + queue if self.admin \
+            else '/orders/' + queue
+        r = requests.get(self.server + endpoint,
+                         params={
+                             'channel': channel,
+                             'limit': limit
+                         },
+                         headers={'X-Auth-Token': self.auth_token},
+                         cert=(self.tls_cert, self.tls_key))
+
+        if (r.status_code != requests.codes.ok):
+            self._print_errors(r)
+
+        r.raise_for_status()
+
+        orders = r.json()
+
+        if status is None:
+            return orders
+
+        filtered_orders = []
+        for order in orders:
+            if order['status'] in status:
+                filtered_orders.append(order)
+
+        return filtered_orders
+
+    def send(self, data, bid, regions=None, channel=None):
         """Send the transmission order
 
         Args:
@@ -123,13 +195,18 @@ class ApiOrder:
         """
         assert (isinstance(data, bytes))
 
-        req_data = {'bid': bid}
+        req_data = {}
 
+        if bid is not None:
+            req_data['bid'] = bid
         if (regions is not None and len(regions) > 0):
             req_data['regions'] = json.dumps(regions)
+        if (channel is not None):
+            req_data['channel'] = channel
 
         # Post request to the API
-        r = requests.post(self.server + '/order',
+        endpoint = '/admin/order' if self.admin else '/order'
+        r = requests.post(self.server + endpoint,
                           data=req_data,
                           files={'file': data},
                           cert=(self.tls_cert, self.tls_key))
@@ -166,7 +243,8 @@ class ApiOrder:
         """Get data content sent over an API order"""
         logger.debug("Fetch message #%s from API" % (self.seq_num))
 
-        r = requests.get(self.server + '/message/' + str(self.seq_num),
+        endpoint = '/admin/message/' if self.admin else '/message/'
+        r = requests.get(self.server + endpoint + str(self.seq_num),
                          cert=(self.tls_cert, self.tls_key))
 
         r.raise_for_status()
@@ -399,7 +477,8 @@ class ApiOrder:
             self._fetch()
 
         # Post delete request
-        r = requests.delete(self.server + '/order/' + self.uuid,
+        endpoint = '/admin/order/' if self.admin else '/order/'
+        r = requests.delete(self.server + endpoint + self.uuid,
                             headers={'X-Auth-Token': self.auth_token},
                             cert=(self.tls_cert, self.tls_key))
 
