@@ -2,7 +2,6 @@
 import logging
 import struct
 import time
-from enum import Enum
 from math import ceil
 
 logger = logging.getLogger(__name__)
@@ -27,15 +26,6 @@ UDP_IP_HEADER = 20 + 8
 MAX_PAYLOAD = 1500 - (UDP_IP_HEADER + HEADER_LEN)
 
 
-class ApiChannel(Enum):
-    ALL = 0
-    USER = 1
-    CTRL = 2
-    AUTH = 3
-    GOSSIP = 4
-    BTC_SRC = 5
-
-
 class BlocksatPkt:
     """Blocksat Packet
 
@@ -67,7 +57,7 @@ class BlocksatPkt:
                  more_frags=None,
                  payload=None):
         if (chan_num is not None):
-            assert(chan_num >= 0 and chan_num < 256), \
+            assert (chan_num >= 0 and chan_num < 256), \
                 "Channel number must be >=0 && < 256"
         self.seq_num = seq_num
         self.frag_num = frag_num
