@@ -1,17 +1,10 @@
 import os
-import shutil
-import unittest
-import uuid
 from .gpg import Gpg
 
+from ..test_helpers import TestEnv
 
-class TestGpg(unittest.TestCase):
 
-    def setUp(self):
-        self.gpghome = "/tmp/.gnupg-" + str(uuid.uuid4())
-
-    def tearDown(self):
-        shutil.rmtree(self.gpghome, ignore_errors=True)
+class TestGpg(TestEnv):
 
     def test_key_creation(self):
         """Test creation of GPG keys"""
@@ -82,3 +75,20 @@ class TestGpg(unittest.TestCase):
 
         # Check
         self.assertEqual(data, dec_data)
+
+    def test_passphrase_validation(self):
+        name = "Test"
+        email = "test@test.com"
+        comment = "comment"
+        passphrase = "test"
+        gpg = Gpg(self.gpghome)
+        gpg.create_keys(name, email, comment, passphrase)
+
+        fingerprint = gpg.get_default_priv_key()['fingerprint']
+        self.assertTrue(gpg.test_passphrase(fingerprint))
+
+        gpg.passphrase = 'wrong-passphrase'
+        self.assertFalse(gpg.test_passphrase(fingerprint))
+
+        gpg.passphrase = None
+        self.assertFalse(gpg.test_passphrase(fingerprint))
