@@ -23,9 +23,9 @@ def _cfg_satellite():
                "https://blockstream.com/satellite/#satellite_network-coverage"
 
     question = "Which satellite below covers your location?"
-    sat = util.ask_multiple_choice(
-        defs.satellites, question, "Satellite",
-        lambda sat: '{} ({})'.format(sat['name'], sat['alias']), help_msg)
+    sat = util.ask_multiple_choice(defs.satellites, question, "Satellite",
+                                   lambda sat: get_satellite_name(sat),
+                                   help_msg)
     return sat
 
 
@@ -54,13 +54,19 @@ def _get_antenna_name(x):
     return s
 
 
-def _ask_antenna():
+def _ask_antenna(sat):
     """Configure antenna"""
     os.system('clear')
     util.print_header("Antenna")
 
     question = "What kind of antenna are you using?"
-    antenna = util.ask_multiple_choice(defs.antennas,
+
+    # The flat-panel antenna models do not work in the C band
+    antenna_options = [
+        x for x in defs.antennas
+        if not (sat["band"] == "C" and x["type"] in ["flat", "sat-ip"])
+    ]
+    antenna = util.ask_multiple_choice(antenna_options,
                                        question,
                                        "Antenna",
                                        _get_antenna_name,
@@ -76,24 +82,34 @@ def _ask_antenna():
     return antenna
 
 
-def _cfg_rx_setup():
+def _ask_demod(sat):
+    """Ask the receiver/demodulator model"""
+    question = "Select your DVB-S2 receiver:"
+
+    rx_options = [
+        rx for rx in defs.demods
+        # If band C, remove Blockstrem Base Station from option list
+        if not (sat["band"] == "C" and rx["vendor"] == "Selfsat")
+    ]
+
+    setup = util.ask_multiple_choice(
+        rx_options, question, "Receiver",
+        lambda x: '{} receiver'.format(_get_rx_marketing_name(x)))
+
+    return setup
+
+
+def _cfg_rx_setup(sat):
     """Configure Rx setup - which receiver user is using """
     os.system('clear')
     util.print_header("Receiver")
 
-    # Receiver
-    question = "Select your DVB-S2 receiver:"
-    setup = util.ask_multiple_choice(
-        defs.demods, question, "Receiver",
-        lambda x: '{} receiver'.format(_get_rx_marketing_name(x)))
+    # The setup includes the demodulator info, the antenna, and extra fields
+    setup = _ask_demod(sat)
 
     # Network interface connected to the standalone receiver
     if (setup['type'] == defs.standalone_setup_type):
-        try:
-            devices = os.listdir('/sys/class/net/')
-        except FileNotFoundError:
-            devices = None
-            pass
+        devices = util.get_network_interfaces()
 
         question = "Which network interface is connected to the receiver?"
         if (devices is not None):
@@ -122,18 +138,28 @@ def _cfg_rx_setup():
         for antenna in defs.antennas:
             if antenna['type'] == 'sat-ip':
                 setup['antenna'] = antenna
-                return setup
-        # We should have returned in the preceding line.
-        raise RuntimeError("Failed to find antenna of {} receiver".format(
-            defs.sat_ip_setup_type))
+                break
+        if 'antenna' not in setup:
+            raise RuntimeError("Failed to find antenna of {} receiver".format(
+                defs.sat_ip_setup_type))
+
+        custom_ip = util.ask_yes_or_no(
+            "Does the {} have a static IP address?".format(
+                _get_rx_marketing_name(setup)),
+            default="n")
+        if (custom_ip):
+            ipv4_addr = util.typed_input("Which IPv4 address?",
+                                         in_type=IPv4Address)
+            setup["ip_addr"] = str(ipv4_addr)
+        return setup
 
     # Antenna
-    setup['antenna'] = _ask_antenna()
+    setup['antenna'] = _ask_antenna(sat)
 
     return setup
 
 
-def _ask_lnb_freq_range():
+def _ask_lnb_freq_range(sat):
     """Prompt the user for the LNB's input frequency range"""
     in_range = []
     while (True):
@@ -157,6 +183,13 @@ def _ask_lnb_freq_range():
             print("Please, provide the frequencies in MHz.")
             continue
 
+        if not _sat_freq_in_lnb_range(sat, {'in_range': in_range}):
+            print(
+                f"Please, choose a frequency range covering the {sat['name']} "
+                f"downlink frequency of {sat['dl_freq']} MHz")
+            in_range = []
+            continue
+
         break
 
     return in_range
@@ -202,6 +235,22 @@ def _ask_lnb_lo(single_lo=True):
     return lo_freq
 
 
+def _ask_lnb_polarization():
+    question = "Choose the LNB polarization:"
+    options = [{
+        'id': "Dual",
+        'text': "Dual polarization (horizontal and vertical)"
+    }, {
+        'id': "H",
+        'text': "Horizontal"
+    }, {
+        'id': "V",
+        'text': "Vertical"
+    }]
+    return util.ask_multiple_choice(options, question, "Polarization",
+                                    lambda x: '{}'.format(x['text']))
+
+
 def _cfg_custom_lnb(sat):
     """Configure custom LNB based on user-entered specs
 
@@ -235,7 +284,7 @@ def _cfg_custom_lnb(sat):
                                    "from 10.7 to 12.75 GHz?")):
                 custom_lnb_in_range = [10700.0, 12750.0]
             else:
-                custom_lnb_in_range = _ask_lnb_freq_range()
+                custom_lnb_in_range = _ask_lnb_freq_range(sat)
 
             # LO
             print()
@@ -249,28 +298,16 @@ def _cfg_custom_lnb(sat):
                 custom_lnb_lo_freq = _ask_lnb_lo(single_lo=False)
         else:
             # Non-universal Ku-band LNB
-            custom_lnb_in_range = _ask_lnb_freq_range()
+            custom_lnb_in_range = _ask_lnb_freq_range(sat)
             custom_lnb_lo_freq = _ask_lnb_lo()
     else:
         # C-band LNB
         custom_lnb_universal = False
-        custom_lnb_in_range = _ask_lnb_freq_range()
+        custom_lnb_in_range = _ask_lnb_freq_range(sat)
         custom_lnb_lo_freq = _ask_lnb_lo()
 
     # Polarization
-    question = "Choose the LNB polarization:"
-    options = [{
-        'id': "Dual",
-        'text': "Dual polarization (horizontal and vertical)"
-    }, {
-        'id': "H",
-        'text': "Horizontal"
-    }, {
-        'id': "V",
-        'text': "Vertical"
-    }]
-    pol = util.ask_multiple_choice(options, question, "Polarization",
-                                   lambda x: '{}'.format(x['text']))
+    pol = _ask_lnb_polarization()
 
     return {
         'vendor': "",
@@ -413,9 +450,9 @@ def _cfg_frequencies(sat, lnb, setup):
 
     if (sat['band'].lower() == "ku"):
         if (lnb['universal']):
-            assert(isinstance(lnb['lo_freq'], list)), \
+            assert (isinstance(lnb['lo_freq'], list)), \
                 "A Universal LNB must have a list with two LO frequencies"
-            assert(len(lnb['lo_freq']) == 2), \
+            assert (len(lnb['lo_freq']) == 2), \
                 "A Universal LNB must have two LO frequencies"
 
             if (sat['dl_freq'] > defs.ku_band_thresh):
@@ -442,7 +479,7 @@ def _cfg_frequencies(sat, lnb, setup):
     return {'dl': sat['dl_freq'], 'lo': lo_freq, 'l_band': if_freq}
 
 
-def _cfg_chan_conf(info, chan_file):
+def _cfg_chan_conf(info, chan_file, yes=False):
     """Generate the channels.conf file"""
 
     util.print_header("Channel Configuration")
@@ -455,11 +492,11 @@ def _cfg_chan_conf(info, chan_file):
     if (os.path.isfile(chan_file)):
         print("Found previous %s file:" % (chan_file))
 
-        if (not util.ask_yes_or_no("Remove and regenerate file?")):
+        if (yes or util.ask_yes_or_no("Remove and regenerate file?")):
+            os.remove(chan_file)
+        else:
             print("Configuration aborted.")
             return
-        else:
-            os.remove(chan_file)
 
     with open(chan_file, 'w') as f:
         f.write('[blocksat-ch]\n')
@@ -546,6 +583,9 @@ def _patch_cfg_file(cfg_file, info):
         if info['sat']['alias'] == "T11N EU" and \
                 info['sat']['dl_freq'] == 11484.3:
             new_dl_freq = 11505.4
+        if info['sat']['alias'] == "G18" and \
+                info['sat']['dl_freq'] == 12016.4:
+            new_dl_freq = 11913.4
 
         if new_dl_freq is not None:
             logger.info(
@@ -619,6 +659,11 @@ def get_rx_model(user_info):
             user_info['setup']['model']).strip()
 
 
+def get_satellite_name(sat):
+    """Return string with satellite name"""
+    return f"{sat['name']} ({sat['alias']})"
+
+
 def get_antenna_model(user_info):
     """Return string with the antenna model"""
     antenna_info = user_info['setup']['antenna']
@@ -675,7 +720,7 @@ def get_net_if(user_info):
     return interface
 
 
-def subparser(subparsers):
+def subparser(subparsers):  # pragma: no cover
     """Argument parser of config command"""
     p = subparsers.add_parser('config',
                               aliases=['cfg'],
@@ -692,6 +737,10 @@ def subparser(subparsers):
         description="Display the local configuration",
         help='Display the local configuration',
         formatter_class=ArgumentDefaultsHelpFormatter)
+    p2.add_argument('--json',
+                    default=False,
+                    action='store_true',
+                    help='Print results in JSON format')
     p2.set_defaults(func=show)
 
     p3 = subsubparsers.add_parser(
@@ -714,7 +763,7 @@ def configure(args):
 
     try:
         user_sat = _cfg_satellite()
-        user_setup = _cfg_rx_setup()
+        user_setup = _cfg_rx_setup(user_sat)
         user_lnb = _cfg_lnb(user_sat, user_setup)
         user_freqs = _cfg_frequencies(user_sat, user_lnb, user_setup)
     except KeyboardInterrupt:
@@ -754,37 +803,66 @@ def configure(args):
     """)
 
 
+def get_readable_cfg(info):
+    assert (info is not None), "Empty receiver configuration"
+
+    setup = info['setup']
+    sat = info['sat']
+    freqs = info['freqs']
+
+    cfg = {
+        'Setup': {
+            'Receiver': _get_rx_marketing_name(setup),
+            'Tuning range':
+            f"From {setup['tun_range'][0]} to {setup['tun_range'][1]} MHz ",
+            'LNB': get_lnb_model(info),
+            'Antenna': get_antenna_model(info),
+        },
+        'Satellite': {
+            'Satellite name':
+            get_satellite_name(sat),
+            'Signal band':
+            sat['band'],
+            'Signal polarization':
+            "Horizontal" if sat['pol'] == "H" else "Vertical"
+        },
+        'Frequencies': {
+            'Downlink frequency': f"{freqs['dl']} MHz",
+            'LNB LO frequency': f"{freqs['lo']} MHz",
+            'Receiver L-band frequency': f"{freqs['l_band']} MHz"
+        }
+    }
+
+    if (setup['type'] == defs.standalone_setup_type and 'rx_ip' in setup):
+        cfg['Setup']['IP Address'] = setup['rx_ip']
+
+    if (setup['type'] == defs.sat_ip_setup_type and 'ip_addr' in setup):
+        cfg['Setup']['IP Address'] = setup['ip_addr']
+
+    return cfg
+
+
 def show(args):
     """Print the local configuration"""
     info = read_cfg_file(args.cfg, args.cfg_dir)
     if (info is None):
         return
-    print("| {:30s} | {:25s} |".format("Receiver",
-                                       _get_rx_marketing_name(info['setup'])))
-    print("| {:30s} | {:25s} |".format("LNB", get_lnb_model(info)))
-    print("| {:30s} | {:25s} |".format("Antenna", get_antenna_model(info)))
-    pr_cfgs = {
-        'freqs': {
-            'dl': ("Downlink frequency", "MHz", None),
-            'lo': ("LNB LO frequency", "MHz", None),
-            'l_band': ("Receiver L-band frequency", "MHz", None)
-        },
-        'sat': {
-            'name': ("Satellite name", "", None),
-            'band': ("Signal band", "", None),
-            'pol': ("Signal polarization", "", lambda x: "Horizontal"
-                    if x == "H" else "vertical"),
-        }
-    }
+
+    pr_cfgs = get_readable_cfg(info)
+
+    if args.json:
+        print(json.dumps(pr_cfgs, indent=4))
+        return
+
+    box_size = 62
+    box_line = "-" * box_size
+
     for category in pr_cfgs:
+        print(f"\n{category}")
+        print(box_line)
         for key, pr_cfg in pr_cfgs[category].items():
-            label = pr_cfg[0]
-            unit = pr_cfg[1]
-            val = info[category][key]
-            # Transform the value if a callback is defined
-            if (pr_cfg[2] is not None):
-                val = pr_cfg[2](val)
-            print("| {:30s} | {:25s} |".format(label, str(val) + " " + unit))
+            print("| {:30s} | {:25s} |".format(key, pr_cfg))
+        print(box_line)
 
 
 def channel(args):
