@@ -25,6 +25,88 @@ target_map = {
     "sat-ip": defs.sat_ip_setup_type
 }
 supported_distros = ["ubuntu", "debian", "raspbian", "fedora", "centos"]
+pkg_map = {
+    'sdr': {
+        'apt': ["gqrx-sdr", "rtl-sdr", "leandvb", "tsduck"],
+        'dnf': ["gqrx", "rtl-sdr", "leandvb", "tsduck"],
+        'yum': ["rtl-sdr", "leandvb", "tsduck"]
+    },
+    'usb': {
+        'apt': ["iproute2", "iptables", "dvb-apps", "dvb-tools"],
+        'dnf': ["iproute", "iptables", "dvb-apps", "v4l-utils"],
+        'yum': ["iproute", "iptables", "dvb-apps", "v4l-utils"]
+    },
+    'standalone': {
+        'apt': ["iproute2", "iptables"],
+        'dnf': ["iproute", "iptables"],
+        'yum': ["iproute", "iptables"]
+    },
+    'sat-ip': {
+        'apt': ["iproute2", "tsduck"],
+        'dnf': ["iproute", "tsduck"],
+        'yum': ["iproute", "tsduck"]
+    }
+}
+app_map = {
+    'sdr': ["rtl_sdr", "leandvb", "ldpc_tool", "tsp"],
+    'usb': ["dvbv5-zap", "dvbnet", "dvb-fe-tool", "ip", "iptables"],
+    'standalone': ["iptables"],
+    'sat-ip': ["tsp", "ip"]
+}
+supported_pkg_managers = ["apt", "dnf", "yum"]
+
+
+def _get_pkg_manager():
+    """Check the package manager installed on the system"""
+    for manager in supported_pkg_managers:
+        if which(manager):
+            return manager
+
+    raise RuntimeError("Could not find a supported package manager")
+
+
+def _is_package_installed(package, required_version):
+    """Check if the package is installed and its version meets the minimum"""
+    distro_id = distro.id()
+
+    if distro_id == 'ubuntu':
+        cmd = ['apt', 'list', '--installed', package]
+    elif distro_id == 'fedora':
+        cmd = ['dnf', 'list', '--installed', package]
+    else:
+        raise ValueError('Unsupported distribution')
+
+    res = runner.run(cmd, capture_output=True, nocheck=True)
+    if (res.returncode != 0):
+        return False
+
+    lines = res.stdout.decode().splitlines()
+
+    installed_version = None
+    if distro_id == 'ubuntu':
+        for line in lines:
+            if package in line:
+                version_release = line.split(' ')[1]
+                installed_version = version_release.split('-')[0]
+                break
+    elif distro_id == 'fedora':
+        if len(lines) > 1:
+            installed_version = lines[1].split()[1].split('-')[0]
+
+    if installed_version is None:
+        return False
+
+    if (LooseVersion(installed_version) < LooseVersion(required_version)):
+        return False
+
+    return True
+
+
+def _create_cmd(cmd: list, interactive: bool):
+    new_cmd = cmd.copy()
+    if not interactive:
+        new_cmd.append('-y')
+    return new_cmd
 
 
 def _check_distro(setup_type):
@@ -71,7 +153,8 @@ def _check_pkg_repo(distro_id):
 
     """
     found = False
-    if (which("apt")):
+    manager = _get_pkg_manager()
+    if (manager == 'apt'):
         apt_repo = _get_apt_repo(distro_id)
         grep_str = apt_repo.replace("ppa:", "")
         apt_sources = glob.glob("/etc/apt/sources.list.d/*")
@@ -80,20 +163,18 @@ def _check_pkg_repo(distro_id):
         cmd.extend(apt_sources)
         res = runner.run(cmd, stdout=subprocess.DEVNULL, nocheck=True)
         found = (res.returncode == 0)
-    elif (which("dnf")):
+    elif (manager == 'dnf'):
         res = runner.run(["dnf", "copr", "list", "--enabled"],
                          root=True,
                          capture_output=True)
         pkgs = res.stdout.decode().splitlines()
         found = ("copr.fedorainfracloud.org/blockstream/satellite" in pkgs)
-    elif (which("yum")):
+    elif (manager == 'yum'):
         yum_sources = glob.glob("/etc/yum.repos.d/*")
         cmd = ["grep", "blockstream/satellite"]
         cmd.extend(yum_sources)
         res = runner.run(cmd, stdout=subprocess.DEVNULL, nocheck=True)
         found = (res.returncode == 0)
-    else:
-        raise RuntimeError("Could not find a supported package manager")
 
     if (found):
         logger.debug("blockstream/satellite repository already enabled")
@@ -110,7 +191,8 @@ def _enable_pkg_repo(distro_id, interactive):
 
     """
     cmds = list()
-    if (which("apt")):
+    manager = _get_pkg_manager()
+    if (manager == 'apt'):
         apt_repo = _get_apt_repo(distro_id)
 
         # On debian, add the apt repository through "add-apt-repository". On
@@ -157,18 +239,16 @@ def _enable_pkg_repo(distro_id, interactive):
         if (not interactive):
             cmd.append("-y")
         cmds.append(cmd)
-    elif (which("dnf")):
+    elif (manager == 'dnf'):
         cmd = ["dnf", "copr", "enable", "blockstream/satellite"]
         if (not interactive):
             cmd.append("-y")
         cmds.append(cmd)
-    elif (which("yum")):
+    elif (manager == 'yum'):
         cmd = ["yum", "copr", "enable", "blockstream/satellite"]
         if (not interactive):
             cmd.append("-y")
         cmds.append(cmd)
-    else:
-        raise RuntimeError("Could not find a supported package manager")
 
     for cmd in cmds:
         runner.run(cmd, root=True)
@@ -182,8 +262,8 @@ def _update_pkg_repo(interactive):
     automatically when an install/upgrade command is called.
 
     """
-
-    if (not which("apt")):
+    manager = _get_pkg_manager()
+    if (manager != 'apt'):
         return
 
     cmd = ["apt", "update"]
@@ -197,7 +277,8 @@ def _install_packages(apt_list,
                       dnf_list,
                       yum_list,
                       interactive=True,
-                      update=False):
+                      update=False,
+                      cwd=None):
     """Install binary packages
 
     Args:
@@ -206,31 +287,27 @@ def _install_packages(apt_list,
         dnf_list    : List of package names for installation via yum
         interactive : Whether to run an interactive install (w/ user prompting)
         update      : Whether to update pre-installed packages instead
-
+        cwd         : Directory from which to run the install command
 
     """
-    if (which("apt")):
-        manager = "apt"
-        cmd = ["apt-get", "install"]
+    manager = _get_pkg_manager()
+    if (manager == 'apt'):
+        cmd = ["apt", "install"]
         if (update):
             cmd.append("--only-upgrade")
         cmd.extend(apt_list)
-    elif (which("dnf")):
-        manager = "dnf"
+    elif (manager == 'dnf'):
         if (update):
             cmd = ["dnf", "upgrade"]
         else:
             cmd = ["dnf", "install"]
         cmd.extend(dnf_list)
-    elif (which("yum")):
-        manager = "yum"
+    elif (manager == 'yum'):
         if (update):
             cmd = ["yum", "upgrade"]
         else:
             cmd = ["yum", "install"]
         cmd.extend(yum_list)
-    else:
-        raise RuntimeError("Could not find a supported package manager")
 
     env = None
     if (not interactive):
@@ -239,7 +316,7 @@ def _install_packages(apt_list,
             env = os.environ.copy()
             env["DEBIAN_FRONTEND"] = "noninteractive"
 
-    runner.run(cmd, root=True, env=env)
+    runner.run(cmd, root=True, env=env, cwd=cwd)
 
 
 def _install_common(interactive=True, update=False, btc=False):
@@ -292,31 +369,10 @@ def _install_common(interactive=True, update=False, btc=False):
                               update)
 
 
-def _install_specific(target, interactive=True, update=False):
+def _install_specific(cfg_dir, target, interactive=True, update=False):
     """Install setup-specific dependencies"""
     key = next(key for key, val in target_map.items() if val == target)
-    pkg_map = {
-        'sdr': {
-            'apt': ["gqrx-sdr", "rtl-sdr", "leandvb", "tsduck"],
-            'dnf': ["gqrx", "rtl-sdr", "leandvb", "tsduck"],
-            'yum': ["rtl-sdr", "leandvb", "tsduck"]
-        },
-        'usb': {
-            'apt': ["iproute2", "iptables", "dvb-apps", "dvb-tools"],
-            'dnf': ["iproute", "iptables", "dvb-apps", "v4l-utils"],
-            'yum': ["iproute", "iptables", "dvb-apps", "v4l-utils"]
-        },
-        'standalone': {
-            'apt': ["iproute2", "iptables"],
-            'dnf': ["iproute", "iptables"],
-            'yum': ["iproute", "iptables"]
-        },
-        'sat-ip': {
-            'apt': ["iproute2", "tsduck"],
-            'dnf': ["iproute", "tsduck"],
-            'yum': ["iproute", "tsduck"]
-        }
-    }
+
     # NOTE:
     # - leandvb and tsduck come from our repository. Also, note gqrx is not
     #   available on CentOS (hence not included on the yum list).
@@ -328,6 +384,19 @@ def _install_specific(target, interactive=True, update=False):
     _install_packages(pkg_map[key]['apt'], pkg_map[key]['dnf'],
                       pkg_map[key]['yum'], interactive, update)
 
+    # On Fedora >= 36 or Ubuntu >= 22.04, both of which have GNU Radio 3.10
+    # available on the main package repo, install gr-dvbs2rx:
+    distro_id = distro.id()
+    distro_ver = distro.version()
+    fc36_or_higher = distro_id == 'fedora' and int(distro_ver) >= 36
+    ubuntu22_or_higher = distro_id == 'ubuntu' and LooseVersion(
+        distro_ver) >= '22.04'
+    if target == defs.sdr_setup_type and \
+            (fc36_or_higher or ubuntu22_or_higher):
+        _install_packages(['gr-dvbs2rx', 'gr-osmosdr'],
+                          ['gr-dvbs2rx', 'gr-osmosdr'], [], interactive,
+                          update)
+
 
 def _print_help(args):
     """Re-create argparse's help menu"""
@@ -337,7 +406,7 @@ def _print_help(args):
     print(parser.format_help())
 
 
-def subparser(subparsers):
+def subparser(subparsers):  # pragma: no cover
     """Parser for install command"""
     p = subparsers.add_parser('dependencies',
                               aliases=['deps'],
@@ -394,6 +463,9 @@ def subparser(subparsers):
         action='store_true',
         default=False,
         help="Clean a previous build before rebuilding the drivers")
+    p3.add_argument("--disable",
+                    nargs='+',
+                    help="Linux media module to disable on compilation")
     p3.set_defaults(func=drivers)
 
     return p
@@ -432,7 +504,10 @@ def run(args):
     _install_common(interactive=interactive, update=args.update, btc=args.btc)
 
     # Install setup-specific dependencies
-    _install_specific(target, interactive=interactive, update=args.update)
+    _install_specific(args.cfg_dir,
+                      target,
+                      interactive=interactive,
+                      update=args.update)
 
 
 def drivers(args):
@@ -472,7 +547,8 @@ def drivers(args):
     # On dnf, not always the kernel-devel/headers package will be available for
     # the same version as the current kernel. Check:
     dnf_update_required = False
-    if (which("dnf")):
+    pkg_manager = _get_pkg_manager()
+    if (pkg_manager == 'dnf'):
         res_d = runner.run(["dnf", "list", kernel_devel],
                            stdout=subprocess.DEVNULL,
                            stderr=subprocess.DEVNULL,
@@ -568,7 +644,7 @@ def drivers(args):
         os.makedirs(driver_src_dir)
 
     if os.path.exists(media_build_dir):
-        runner.run(["git", "pull", "origin", "master"], cwd=media_build_dir)
+        runner.run(["git", "pull", "origin"], cwd=media_build_dir)
     else:
         runner.run(
             ["git", "clone", "https://github.com/tbsdtv/media_build.git"],
@@ -592,7 +668,11 @@ def drivers(args):
         runner.run(["make", "cleanall"], cwd=media_build_dir)
 
     runner.run(["make", "dir", "DIR=../media"], cwd=media_build_dir)
+
+    # Enable all modules but set up a list with some distro-specific and
+    # user-requested modules to be disabled
     runner.run(["make", "allyesconfig"], cwd=media_build_dir)
+    disable_list = []
 
     # FIXME: Temporary workaround for error "modpost: "__devm_regmap_init_sccb"
     # ov9650.ko undefined!": disable ov9650 from the build. The problem was
@@ -600,11 +680,7 @@ def drivers(args):
     # version < 5.8.
     if (distro_id == "fedora"
             and LooseVersion(linux_release) < LooseVersion('5.8')):
-        runner.run([
-            "sed", "-i", "s/CONFIG_VIDEO_OV9650=m/CONFIG_VIDEO_OV9650=n/g",
-            "v4l/.config"
-        ],
-                   cwd=media_build_dir)
+        disable_list.append("VIDEO_OV9650")
 
     # On Raspbian, disable RC/IR support and disable the MN88436 drivers to
     # avoid the __aeabi_ldivmod/__aeabi_uldivmod undefined errors. Also,
@@ -618,14 +694,15 @@ def drivers(args):
         runner.run(
             ["sed", "-i", "-r", "s/(^CONFIG.*_IR.*=)./\1n/g", "v4l/.config"],
             cwd=media_build_dir)
+        disable_list.append("DVB_MN88436")
+
+    if args.disable is not None:
+        disable_list.extend(args.disable)
+
+    for module in disable_list:
+        logger.debug("Disabling module {}".format(module))
         runner.run([
-            "sed", "-i", "s/CONFIG_DVB_MN88436=m/CONFIG_DVB_MN88436=n/g",
-            "v4l/.config"
-        ],
-                   cwd=media_build_dir)
-        runner.run([
-            "sed", "-i",
-            "s/CONFIG_VIDEO_SAA7146_VV=m/CONFIG_VIDEO_SAA7146_VV=n/g",
+            "sed", "-i", "s/CONFIG_{0}=m/CONFIG_{0}=n/g".format(module),
             "v4l/.config"
         ],
                    cwd=media_build_dir)
@@ -667,3 +744,67 @@ def check_apps(apps):
             return False
     # All apps are installed
     return True
+
+
+def get_pkg_list(target):
+    """Get the list of package dependencies for a specific receiver and OS
+
+    Args:
+        target (str): DVB-S2 receiver ('sdr', 'usb', 'standalone' or 'sat-ip')
+
+    Returns:
+        pkg_list (list): Package list
+
+    """
+    assert (target in pkg_map.keys()), f"Unsupported target {target}"
+    manager = _get_pkg_manager()
+    return pkg_map[target][manager]
+
+
+def check_dependencies(target):
+    """Check if the package dependency list is already installed
+
+    Args:
+        target: DVB-S2 receiver
+
+    Returns:
+        True if all dependencies are installed or False otherwise.
+
+    """
+    assert (target in app_map.keys())
+    return check_apps(app_map[target])
+
+
+def check_drivers(model=None):
+    """Check if the USB driver is already installed
+
+    Args:
+        model: USB receiver model.
+
+    Returns:
+        True if the drivers are installed or False otherwise.
+
+    """
+    module_map = {
+        "5927": "dvb-usb-tbs5927.ko",
+        "5520SE": "dvb-usb-tbs5520se.ko"
+    }
+    if model is None:
+        models = list(module_map.keys())
+    else:
+        assert (model in module_map.keys())
+        models = [model]
+
+    release = platform.release()
+    for model in models:
+        modules_file = f"/lib/modules/{release}/modules.dep"
+        if not os.path.exists(modules_file):
+            logger.warning(
+                f"Unable to check USB drivers. {modules_file} is missing.")
+            return False
+
+    # Look for the USB modules in the modules.dep file.
+    with open(modules_file, "r") as f:
+        kernel_modules = f.read()
+
+    return module_map[model] in kernel_modules
