@@ -131,7 +131,7 @@ def _add_iptables_rule(net_if, cmd):
 
 
 def _configure_iptables(net_if, ports, igmp=False, prompt=True):
-    """Configure iptables rules to accept blocksat traffic on a DVB-S2 interface
+    """Configure iptables rules on DVB-S2 interface
 
     Args:
         net_if : DVB network interface name
@@ -325,7 +325,7 @@ def configure(net_ifs, ports, src_ip, igmp=False, prompt=True, dry=False):
             print("")
 
 
-def subparser(subparsers):
+def subparser(subparsers):  # pragma: no cover
     """Parser for firewall command"""
     p = subparsers.add_parser('firewall',
                               description="Set firewall rules",
