@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 
-__version__ = "0.4.4"
+__version__ = "0.4.5"
 
 import logging
 import os
@@ -9,13 +9,11 @@ import time
 from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter
 
 from . import config, util, instructions, gqrx, bitcoin, sdr, rp, \
-    firewall, standalone, usb, satip, dependencies, update
+    firewall, standalone, usb, satip, dependencies, update, monitoring_api
 from .api import api
 
 
-def main():
-    """Main - parse command-line arguments and call subcommands
-    """
+def get_parser():  # pragma: no cover
     default_cfg_dir = os.path.join(util.get_home_dir(), ".blocksat")
 
     parser = ArgumentParser(
@@ -56,12 +54,20 @@ def main():
     sdr.subparser(subparsers)
     api.subparser(subparsers)
     satip.subparser(subparsers)
+    monitoring_api.subparser(subparsers)
 
+    return parser
+
+
+def main():  # pragma: no cover
+    """Main - parse command-line arguments and call subcommands
+    """
+    parser = get_parser()
     args = parser.parse_args()
 
     # Filter commands that are Linux-only
     if (args.subcommand not in ["cfg", "instructions", "api", "btc"]):
-        assert(platform.system() == 'Linux'), \
+        assert (platform.system() == 'Linux'), \
             "Command {} is currently Linux-only".format(args.subcommand)
 
     # Logging
