@@ -3,16 +3,54 @@ import json
 import logging
 import math
 import os
-import requests
 import sys
 import threading
 import time
+from enum import Enum
 from http.server import HTTPServer, BaseHTTPRequestHandler
 
+import requests
+
 from . import defs, config, monitoring_api
 
 logger = logging.getLogger(__name__)
+
 sats = [x['alias'] for x in defs.satellites]
+# Supported receiver metrics, with their labels and printing formats
+rx_metrics = {
+    'lock': {
+        'label': 'Lock',
+        'format_str': ''
+    },
+    'level': {
+        'label': 'Level',
+        'format_str': '.2f'
+    },
+    'snr': {
+        'label': 'SNR',
+        'format_str': '.2f'
+    },
+    'ber': {
+        'label': 'BER',
+        'format_str': '.2e'
+    },
+    'fer': {
+        'label': 'FER',
+        'format_str': '.2e'
+    },
+    'per': {
+        'label': 'PER',
+        'format_str': '.2e'
+    },
+    'quality': {
+        'label': 'Signal Quality',
+        'format_str': '.1f'
+    },
+    'pkt_err': {
+        'label': 'Packet Errors',
+        'format_str': 'd'
+    }
+}
 
 
 def get_report_opts(args):
@@ -29,6 +67,11 @@ def get_report_opts(args):
     }
 
 
+class ReportStatus(Enum):
+    REGISTRATION_RUNNING = 1
+    REGISTRATION_FAILED = 2
+
+
 class Reporter():
     """Receiver Reporter
 
@@ -44,7 +87,8 @@ class Reporter():
                  tls_cert=None,
                  tls_key=None,
                  gnupghome=None,
-                 passphrase=None):
+                 passphrase=None,
+                 reset_api_pwd=False):
         """Reporter Constructor
 
         Args:
@@ -59,6 +103,7 @@ class Reporter():
             passphrase : Passphrase to the private key used when reporting to
                          Blockstream's monitoring API. If None, it will be
                          obtained by prompting the user.
+            reset_api_pwd : Reset password for the monitoring API.
 
         """
         info = config.read_cfg_file(cfg, cfg_dir)
@@ -81,17 +126,21 @@ class Reporter():
         # interaction with this API (e.g., registration).
         if (dest_addr == monitoring_api.metric_endpoint):
             self.bs_monitoring = monitoring_api.BsMonitoring(
-                cfg, cfg_dir, gnupghome, passphrase)
+                cfg, cfg_dir, gnupghome, passphrase, reset_api_pwd)
         else:
             self.bs_monitoring = None
-
-        logger.info("Reporting Rx status to {} ".format(self.dest_addr))
+            logger.info("Reporting Rx status to {} ".format(self.dest_addr))
 
     def send(self, metrics):
         """Save measurement on database
 
         Args:
             metrics : Dictionary with receiver metrics to send over the report
+
+        Returns:
+            int: The HTTP request status code when the request is processed
+            (even on HTTP error), or the ReportStatus value on other failures.
+
         """
         # When the receiver is not registered with the monitoring API, the
         # sign-up procedure runs. However, this procedure can only work if the
@@ -111,28 +160,36 @@ class Reporter():
                 print()
                 logger.error("Report failed: registration incomplete "
                              "(relaunch receiver and try again)")
+                status = ReportStatus.REGISTRATION_FAILED.value
+            else:
+                status = ReportStatus.REGISTRATION_RUNNING.value
 
             # Don't send reports to the monitoring API until the receiver is
             # properly registered and verified
+            return status
+
+        if self.bs_monitoring and self.bs_monitoring.disabled:
             return
 
         data = {}
         data.update(metrics)
 
-        # When reporting to Blockstream's Monitoring API, sign every set of
-        # reported data using the local GPG key and don't send the satellite
+        logger.debug("Report {} to {}".format(data, self.dest_addr))
+
+        # When reporting to Blockstream's Monitoring API, there are two
+        # possibilities to validate the report: using the receiver monitoring
+        # password or signing the reported data with the local GPG key.
+        # Also, if reporting to Blockstream's API, don't send the satellite
         # nor the hostname on the requests. This information is already
         # associated with the account registered with the Monitoring API. In
         # contrast, when reporting to a general-purpose server, do include the
         # satellite and hostname information if defined.
         if (self.bs_monitoring is None):
-            data["satellite"] = self.satellite,
+            data["satellite"] = self.satellite
             if (self.hostname):
                 data['hostname'] = self.hostname
         else:
-            self.bs_monitoring.sign_report(data)
-
-        logger.debug("Report {} to {}".format(data, self.dest_addr))
+            self.bs_monitoring.sign_request(data, password_allowed=True)
 
         try:
             r = requests.post(self.dest_addr,
@@ -141,9 +198,21 @@ class Reporter():
             if (r.status_code != requests.codes.ok):
                 print()
                 logger.error("Report failed: " + r.text)
-        except requests.exceptions.ConnectionError as e:
+            post_status = r.status_code
+        except requests.exceptions.HTTPError as errh:
+            post_status = errh
+        except requests.exceptions.ConnectionError as errc:
+            post_status = errc
+        except requests.exceptions.Timeout as errt:
+            post_status = errt
+        except requests.exceptions.RequestException as err:
+            post_status = err
+
+        if isinstance(post_status, requests.exceptions.RequestException):
             print()
-            logger.error("Report failed: " + str(e))
+            logger.error("Report failed: " + str(post_status))
+
+        return post_status
 
 
 class Server(BaseHTTPRequestHandler):
@@ -181,7 +250,8 @@ class Monitor():
                  port=defs.monitor_port,
                  report=False,
                  report_opts={},
-                 utc=True):
+                 utc=True,
+                 callback=None):
         """Monitor Constructor
 
         Args:
@@ -199,8 +269,13 @@ class Monitor():
                            a remote address.
             report_opts  : Reporter options.
             utc          : Whether to print logs in UTC time.
+            callback     : Callback to process the monitoring updates. It
+                must be a callable with two arguments, the first expecting the
+                receiver status dictionary, the second expecting the report
+                status, if any.
 
         """
+        self.disable_event = threading.Event()
         self.cfg_dir = cfg_dir
         self.logfile = None
         self.scroll = scroll
@@ -208,36 +283,11 @@ class Monitor():
         self.min_interval = min_interval
         self.report = report
         self.utc = utc
+        self.callback = callback
+
         if (logfile):
             self._setup_logfile()
 
-        # Supported receiver metrics, with their labels and printing formats
-        self._metrics = {
-            'lock': {
-                'label': 'Lock',
-                'format_str': ''
-            },
-            'level': {
-                'label': 'Level',
-                'format_str': '.2f'
-            },
-            'snr': {
-                'label': 'SNR',
-                'format_str': '.2f'
-            },
-            'ber': {
-                'label': 'BER',
-                'format_str': '.2e'
-            },
-            'quality': {
-                'label': 'Signal Quality',
-                'format_str': '.1f'
-            },
-            'pkt_err': {
-                'label': 'Packet Errors',
-                'format_str': 'd'
-            }
-        }
         self.stats = {}
 
         # Reporter sessions
@@ -271,14 +321,14 @@ class Monitor():
 
         string = "{} ".format(t_now)
 
-        for key in self._metrics.keys():
+        for key in rx_metrics.keys():
             if key in self.stats:
                 val = self.stats[key]
                 # Label=Value
                 string += " {} = {:{fmt_str}}".format(
-                    self._metrics[key]['label'],
+                    rx_metrics[key]['label'],
                     val[0],
-                    fmt_str=self._metrics[key]['format_str'],
+                    fmt_str=rx_metrics[key]['format_str'],
                 )
                 # Unit
                 if (not math.isnan(val[0]) and val[1]):
@@ -329,7 +379,7 @@ class Monitor():
         assert (all([isinstance(x, tuple) for x in data.values()]))
 
         # All keys in the input dictionary must be supported receiver metrics
-        assert (all([k in self._metrics for k in data.keys()]))
+        assert (all([k in rx_metrics for k in data.keys()]))
 
         # Copy all the data
         self.stats = data
@@ -347,8 +397,10 @@ class Monitor():
                 fd.write(str(self) + "\n")
 
         # Report over HTTP to a remote address
+        report_status = None
+        disable_echo = False  # whether to temporarily disable echoing
         if (self.report):
-            self.reporter.send(self.get_stats())
+            report_status = self.reporter.send(self.get_stats())
             # If the reporter object is configured to report to the Monitoring
             # API but, at this point, it is still waiting for registration to
             # complete, don't log the status to the console yet. Otherwise, the
@@ -367,17 +419,26 @@ class Monitor():
             if (self.reporter.bs_monitoring is not None
                     and not self.reporter.bs_monitoring.registered
                     and self.reporter.bs_monitoring.registration_running):
-                return
+                disable_echo = True
+            else:
+                disable_echo = False
 
         # Print to console
-        if (self.echo):
+        if (self.echo and not disable_echo):
             print_end = '\n' if self.scroll else '\r'
             if (not self.scroll):
                 sys.stdout.write("\033[K")
             print(str(self), end=print_end)
 
+        if self.callback:
+            self.callback(self.get_stats(), report_status)
 
-def add_to_parser(parser):
+    def stop(self):
+        """Stop the monitoring activity"""
+        self.disable_event.set()
+
+
+def add_to_parser(parser):  # pragma: no cover
     """Add receiver monitoring options to parser"""
     m_p = parser.add_argument_group('receiver logging options')
     m_p.add_argument(
@@ -414,7 +475,7 @@ def add_to_parser(parser):
         '--report-dest',
         default=monitoring_api.metric_endpoint,
         help='Destination address in http://ip:port format. By default, '
-        'report to Blockstream\'s Satellite Monitoring API')
+        'report to the Blockstream Satellite Monitoring API')
     r_p.add_argument('--report-hostname', help='Reporter\'s hostname')
     r_p.add_argument(
         '--report-cert',
@@ -424,18 +485,5 @@ def add_to_parser(parser):
         '--report-key',
         default=None,
         help="Private key for client-side authentication with the destination")
-    r_p.add_argument(
-        '--report-gnupghome',
-        default=".gnupg",
-        help="GnuPG home directory, by default created inside the config "
-        "directory specified via --cfg-dir option. This option is used when "
-        "reporting to Blockstream's Satellite Monitoring API only, where it "
-        "determines the name of the directory (inside --cfg-dir) that holds "
-        "the key for authentication with the API")
-    r_p.add_argument(
-        '--report-passphrase',
-        default=None,
-        help="Passphrase to the private GnuPG key used to sign receiver "
-        "status reports sent to Blockstream's Satellite Monitoring API. If "
-        "undefined (default), the program prompts for this passphrase instead."
-    )
+
+    monitoring_api.add_to_parser(parser)
