@@ -617,8 +617,7 @@ def _get_monitor(args):
     """Create an object of the Monitor class
 
     Args:
-        args      : SDR parser arguments
-        sat_name  : Satellite name
+        args : Parser arguments.
 
     """
     # Force scrolling logs if tsp is configured to print to stdout or if
@@ -636,9 +635,9 @@ def _get_monitor(args):
                               utc=args.utc)
 
 
-def _monitoring_thread(sat_ip, handler):
+def _monitoring_thread(sat_ip, handler: monitoring.Monitor):
     """Loop used to monitor the DVB-S2 frontend on a thread"""
-    while (True):
+    while (not handler.disable_event.is_set()):
         time.sleep(1)
 
         status = sat_ip.fe_stats()
@@ -652,7 +651,7 @@ def _monitoring_thread(sat_ip, handler):
         handler.update(status)
 
 
-def subparser(subparsers):
+def subparser(subparsers):  # pragma: no cover
     """Parser for sdr command"""
     p = subparsers.add_parser('sat-ip',
                               description="Launch a Sat-IP receiver",
@@ -706,19 +705,19 @@ def subparser(subparsers):
     return p
 
 
-def launch(args):
+def launch(args, monitor: monitoring.Monitor = None):
     info = config.read_cfg_file(args.cfg, args.cfg_dir)
     if (info is None):
         return
 
     util.check_configured_setup_type(info, defs.sat_ip_setup_type, logger)
 
-    if (not dependencies.check_apps(['tsp', 'ip'])):
+    if (not dependencies.check_dependencies('sat-ip')):
         return
 
     # Discover or define the IP address to communicate with the Sat-IP server
     sat_ip = SatIp()
-    if args.addr is None:
+    if args.addr is None and 'ip_addr' not in info['setup']:
         sat_ip.discover(src_port=args.ssdp_src_port,
                         interface=args.ssdp_net_if)
         if (sat_ip.host is None):
@@ -727,7 +726,7 @@ def launch(args):
                         "through option -a/--addr")
             sys.exit(1)
     else:
-        addr = args.addr
+        addr = args.addr if args.addr is not None else info['setup']['ip_addr']
         try:
             ip_address(addr)
         except ValueError as e:
@@ -790,7 +789,8 @@ def launch(args):
 
     # Launch the monitoring thread
     if (args.fe_monitoring):
-        monitor = _get_monitor(args)
+        if monitor is None:
+            monitor = _get_monitor(args)
         t = threading.Thread(target=_monitoring_thread,
                              args=(sat_ip, monitor),
                              daemon=True)
