@@ -288,7 +288,7 @@ def _dvbnet(adapter, ifnames, pids, ule=False):
     """
     assert (isinstance(ifnames, list))
     assert (isinstance(pids, list))
-    assert(len(ifnames) == len(pids)), \
+    assert (len(ifnames) == len(pids)), \
         "Interface names and PID number must be vectors of the same length"
 
     # Find the dvbnet interfaces that already exist for the chosen adapter
@@ -433,7 +433,7 @@ def zap(adapter,
         cmd = cmd + ["-t", timeout]
 
     if (monitor):
-        assert(output is None), \
+        assert (output is None), \
             "Monitor mode does not work if recording (i.e. w/ -r flag)"
         cmd.append("-m")
 
@@ -457,7 +457,7 @@ def zap(adapter,
     return ps
 
 
-def subparser(subparsers):
+def subparser(subparsers):  # pragma: no cover
     """Subparser for usb command"""
     p = subparsers.add_parser('usb',
                               description="USB DVB-S2 receiver manager",
@@ -654,6 +654,11 @@ def usb_config(args):
     # Configure the subprocess runner
     runner.set_dry(args.dry_run)
 
+    # Confirm the drivers are installed
+    if not args.dry_run and not dependencies.check_drivers(
+            user_info['setup']['model']):
+        return
+
     if args.ip is None:
         ips = ip.compute_rx_ips(user_info['sat']['ip'], len(args.pid))
     else:
@@ -662,7 +667,7 @@ def usb_config(args):
     assert (all(["/" in x
                  for x in ips])), "Please provide IPs in CIDR notation"
 
-    assert(len(args.pid) == len(ips)), \
+    assert (len(args.pid) == len(ips)), \
         "Please define one IP address for each PID."
 
     # dvbnet interfaces of interest
@@ -759,20 +764,41 @@ def _parse_log(line):
     return d
 
 
-def launch(args):
+def _get_monitor(args):
+    """Create an object of the Monitor class
+
+    Args:
+        args : Parser arguments.
+
+    """
+    return monitoring.Monitor(cfg_dir=args.cfg_dir,
+                              logfile=args.log_file,
+                              scroll=args.log_scrolling,
+                              min_interval=args.log_interval,
+                              server=args.monitoring_server,
+                              port=args.monitoring_port,
+                              report=args.report,
+                              report_opts=monitoring.get_report_opts(args),
+                              utc=args.utc)
+
+
+def launch(args, monitor: monitoring.Monitor = None):
     """Launch the DVB interface from scratch
 
     Handles the launch subcommand
 
     """
-
     common_params = _common(args)
     if (common_params is None):
         return
     user_info, adapter, frontend = common_params
 
     # Check if all dependencies are installed
-    if (not dependencies.check_apps(["dvbv5-zap"])):
+    if not dependencies.check_apps(["dvbv5-zap"]):
+        return
+
+    # Confirm the drivers are installed
+    if not dependencies.check_drivers(user_info['setup']['model']):
         return
 
     # Check conflicting logging options
@@ -781,16 +807,8 @@ def launch(args):
         raise ValueError("Logging options are disabled when running with "
                          "-m/--monitor or -r/--record-file")
 
-    # Log Monitoring
-    monitor = monitoring.Monitor(args.cfg_dir,
-                                 logfile=args.log_file,
-                                 scroll=args.log_scrolling,
-                                 min_interval=args.log_interval,
-                                 server=args.monitoring_server,
-                                 port=args.monitoring_port,
-                                 report=args.report,
-                                 report_opts=monitoring.get_report_opts(args),
-                                 utc=args.utc)
+    if monitor is None:
+        monitor = _get_monitor(args)
 
     # Channel configuration file
     chan_conf = user_info['setup']['channel']
@@ -821,7 +839,7 @@ def launch(args):
     if (args.monitor or args.record_file):
         zap_ps.wait()
     else:
-        while (zap_ps.poll() is None):
+        while (zap_ps.poll() is None and not monitor.disable_event.is_set()):
             line = zap_ps.stderr.readline()
             metrics = _parse_log(line)
             if (metrics is None):
